use Test::More tests => 1;

BEGIN {
    use_ok('Net::LDAP::SimpleServer::ProtocolHandler');
}

diag(
"Testing Net::LDAP::SimpleServer::ProtocolHandler $Net::LDAP::SimpleServer::ProtocolHandler::VERSION"
);
